#ifndef __qfnl_
#define __qfnl_
/*********************************************************************/
/*      Licensed Materials - Property of IBM                         */
/*      "Restricted Materials of IBM"                                */
/*      5748-T15                                                     */
/*      Copyright IBM Corp. 2016, 2018                               */
/*      All Rights Reserved                                          */
/*      US Government Users Restricted Rights - Use, duplication     */
/*      or dislosure restricted by GSA ADP Schedule Contract         */
/*      with IBM Corp.                                               */
/*********************************************************************/
/* This file is provided on an "as is" basis.  There are no          */
/* warranties, express or implied, including the implied warranties  */
/* of merchantability and fitness for a particular purpose.          */
/* IBM has no obligation to provide service, defect correction, or   */
/* any maintenance for the file.  IBM has no obligation to supply    */
/* any updates or enhancements for the file to you even if such are  */
/* or later become available.                                        */
/*********************************************************************/
/* Change History                                                    */
/*  Defect           Date               Description                  */
/*  FFACT           02/01/16            Created                      */
/*  dynami3         05/26/17            Enhanced for Dynamic CPU     */
/*                                       Utlization project (PJ44591)*/
/*  d26352          20180810            Compiler compatibility updates                */
/*                                                                   */
/*********************************************************************/

// TODO: This header file isn't strictly needed, but it's useful
//       for the definitions used for continuous mode and the like

// TODO: Place common header files here, or remove unneeded
#include <tpf/c_tglb.h>   //
#include <tpf/c_eb0eb.h>  //
#include <tpf/i_ecb3.h>   //
#include <tpf/c_idspat.h> //

#include <tpf/tpfeq.h>    // general TPF include
#include <tpf/tpfapi.h>   // general TPF include
#include <tpf/sysapi.h>   // general TPF include
#include <tpf/tpfio.h>    // general TPF include
#include <tpf/c_pi1dt.h>
#include <tpf/serupdt.h>  // for tpf serialized update APIs

#include <stdio.h>
#include <stdlib.h>
#include <string.h>


// #include <ctype.h>
#include <tpf/c_cach.h>   // for Logical Record Cache functions
#include <tpf/c_stcke.h>  // for tpf_STCKE

/**********************************************************************/
/* Definitions                                                        */
/**********************************************************************/

#define DRVR_HDR "QFNL"
#define DRVR_NAME "FFACT"

#define CANNED_DELAY 5   // seconds
#define DEFER_MAX 1024
#define QFNL_TC_BEGIN 1  // First Test Case
#define QFNL_TC_END 2    // Last Test Case:  TODO update it

#define CANNED_FORCE 1
#define CANNED_DEFAULT 0

#define DRVR_SUCCESS 0
#define DRVR_FAILURE -1
#define DRVR_INVALID -2

#define HEX_BASE   16            /* Hex base used in strtoul       */

/* Function pointer typedef, used for modular function writing */
typedef int (*testfp)();

/* Function Parameter Structures */

#define QFNL_RID_SIZE 2
#define QFNL_RCC_SIZE 2
#define QFNL_RTYPE_SIZE 9
#define QFNL_STPOOL_SIZE 5           // length of short term pool section name plus 1 for null

#define QFNL_FARF_STRING_SIZE 17

// The following structure is used to hold parameters on a ZTEST FFACt FIXED BUILD command
#ifndef __FIXED_BUILD
   struct __fixed_build {
     int group;           // integer version of group number
     unsigned int begin_ordinal;
     unsigned int end_ordinal;
     char hex_rid[QFNL_RID_SIZE];
     char rtype[QFNL_RTYPE_SIZE];
     int record_size;     // we need to save this in case we are doing writes without reads
     char groupString[6]; // string version of group number
                          // its length is 6 = size of string 99999 which is 5 plus 1 for null terminator
     char cpuID;          // CPU ID who did build for this group
   };
   typedef struct __fixed_build qfnl_fixed_build_parm_list;
   typedef struct __fixed_build *qfnl_fixed_build_parm_list_Ptr;
#define QFNL_DIR "/usr/"    // This is the file system directory QFNL will use
#define QFNL_PFX "QFNL_FIXED_GROUP"  // This is the prefix to the file name used to store group build information
#define QFNL_LCK "QFNL_LOCK_GROUP"  // This is the prefix to the file name used to serialize builds of the same group
#define __FIXED_BUILD 1
#endif

// The following structure is used to hold parameters on a ZTEST FFACt FIXED START command
#ifndef __FIXED_START
   struct __fixed_start {
     int group;           // integer version of group number to start
     int seed;            // seed ultimately used to randomly access ordinals
     int numberOfReads;   // number of reads per child iteration
     int numberOfWrites;  // number of writes per child iteration
     int numECBs;         // number of ECBs to spawn per iteration
     boolean doLocking;   // should the instance being started do locking? TRUE=yes FALSE=no
     char groupString[6]; // string version of group number
                          // its length is 6 = size of string 99999 which is 5 plus 1 for null terminator
     boolean lowPriority;  // should this instance start as a LOW PRIORITY ECB? //dynami2
     boolean inheritLow;   // should all child ECBs be LOW PRIORITY as well?    //dynami2
     boolean switchable;   // should this instance start as a SWITCHABLE ECB?   //dynami2
     boolean inheritSW;    // should all child ECBs be SWITCHABLE as well?      //dynami2
     boolean useResClass;  // should this instance and its children use a       //dynami2 
                           //      resource class? (currently IBMBATCH)         //dynami2
     boolean useUtilClass;  // should this instance and its children use a      //dynami2 
                            //      utilization clas? (currently IBMBATCH)      //dynami2
     char utilClass[9];    // up to 8-character utilization class (plus null)   //dynami2
   };

   typedef struct __fixed_start qfnl_fixed_start_parm_list;
   typedef struct __fixed_start *qfnl_fixed_start_parm_list_Ptr;

#define __FIXED_START
#endif

// The following structure is used to hold parameters on a ZTEST FFACt FIXED STATUS command
#ifndef __FIXED_STATUS
   struct __fixed_status {
     int group;           // integer version of group number to display status
     char groupString[6]; // string version of group number
                          // its length is 6 = size of string 99999 which is 5 plus 1 for null terminator
   };

   typedef struct __fixed_status qfnl_fixed_status_parm_list;
   typedef struct __fixed_status *qfnl_fixed_status_parm_list_Ptr;

#define __FIXED_STATUS
#endif

// The following structure is used to hold parameters on a ZTEST FFACt FIXED STOP command
#ifndef __FIXED_STOP
   struct __fixed_stop {
     int group;           // integer version of group number to stop
   };

   typedef struct __fixed_stop qfnl_fixed_stop_parm_list;
   typedef struct __fixed_stop *qfnl_fixed_stop_parm_list_Ptr;

#define __FIXED_STOP
#endif

// The following structure is used to hold parameters on a ZTEST FFACt FIXED KILL command
#ifndef __FIXED_KILL
   struct __fixed_kill {
     int group;           // integer version of group number to kill
     char groupString[6]; // string version of group number
                          // its length is 6 = size of string 99999 which is 5 plus 1 for null terminator
   };

   typedef struct __fixed_kill qfnl_fixed_kill_parm_list;
   typedef struct __fixed_kill *qfnl_fixed_kill_parm_list_Ptr;

#define __FIXED_KILL
#endif

// The following structure is used to hold parameters on a ZTEST FFACt FIXED KILL command
#ifndef __FIXED_CLEAR
   struct __fixed_clear {
     int group;           // integer version of group number whose counts should be cleared
   };

   typedef struct __fixed_clear qfnl_fixed_clear_parm_list;
   typedef struct __fixed_clear *qfnl_fixed_clear_parm_list_Ptr;

#define __FIXED_CLEAR
#endif

// The following structure is used to hold the parameters that ZTEST FFACt FIXED START will
//   pass to each child ECB

#ifndef __ECB_PARMS
   typedef enum {
    QFNL_FIXED=1,
    QFNL_POOL=2
   } qfnl_group_type;

   struct __ecb_parms {
     int ecbNum;          // ecb number for printf messages
     int group;           // integer version of group number to start
     int group_type;      // fixed or pool indicator
     char rec_ID[QFNL_RID_SIZE];   // FOR FIXED ONLY:  record ID for this group of fixed records
     int child_seed;      // seed passed to child for use to randomly access ordinals
     int numberOfReads;   // number of reads per child iteration
     int numberOfWrites;  // number of writes per child iteration
     boolean doLocking;   // should the instance being started do locking? TRUE=yes FALSE=no
   };

   typedef struct __ecb_parms qfnl_child_ecb_parm_list;
   typedef struct __ecb_parms *qfnl_child_ecb_parm_list_Ptr;

#define __ECB_PARMS
#endif

// The following structure is used to hold parameters on a ZTEST FFACt POOL BUILD command
#ifndef __POOL_BUILD
   struct __pool_build {
     int group;           // integer version of group number
     unsigned int number_records;
     char hex_rid[QFNL_RID_SIZE];  // used to populate records
     char stpool[QFNL_STPOOL_SIZE];  // short term pool section for this group
     unsigned short getfc_rid;  // RID used by GETFC just to get from the appropriate pool section
     int record_size;     // we need to save this in case we are doing writes without reads
     int expiry; // number of seconds (60*input Expire value in minutes)
     char groupString[6]; // string version of group number
                          // its length is 6 = size of string 99999 which is 5 plus 1 for null terminator
     char cpuID;          // CPU ID who did build for this group
     tpf_TOD_ext_type tod_built;   // time that this pool group was built (used for expiry)
   };
   typedef struct __pool_build qfnl_pool_build_parm_list;
   typedef struct __pool_build *qfnl_pool_build_parm_list_Ptr;
#define __POOL_BUILD 1

// The following are taken from tpfdf/rt/ub20.asm.  Make sure you have them
//   defined on your system before using the ZTEST FFACT POOL driver.  If you do not,
//   you will have to add similar record IDs for your system.

#define ridSSTA 0xfcdc // record ID to get SSTA, etc.
#define ridSSTB 0xfcdf
#define ridSSTC 0xfce2
#define ridSSTD 0xfce5

#define ridLSTA 0xfce8 // record ID to get LSTA, etc.
#define ridLSTB 0xfceb
#define ridLSTC 0xfcee
#define ridLSTD 0xfcf1

#define rid4STA 0xfcf4 // record ID to get 4STA, etc.
#define rid4STB 0xfcf7
#define rid4STC 0xfcfa
#define rid4STD 0xfcfd

#define rid4S6A 0xfcd4
#define rid4S6B 0xfcd5
#define rid4S6C 0xfcd6
#define rid4S6D 0xfcd7

#endif


// The following structure is used to hold parameters on a ZTEST FFACt POOL START command
#ifndef __POOL_START
   struct __pool_start {
     int group;           // integer version of group number to start
     int seed;            // seed ultimately used to randomly access ordinals
     int numberOfReads;   // number of reads per child iteration
     int numberOfWrites;  // number of writes per child iteration
     int numECBs;         // number of ECBs to spawn per iteration
     boolean doLocking;   // should the instance being started do locking? TRUE=yes FALSE=no
     char groupString[6]; // string version of group number
                          // its length is 6 = size of string 99999 which is 5 plus 1 for null terminator
     boolean lowPriority;  // should this instance start as a LOW PRIORITY ECB? //dynami2
     boolean inheritLow;   // should all child ECBs be LOW PRIORITY as well?    //dynami2
     boolean switchable;   // should this instance start as a SWITCHABLE ECB?   //dynami2
     boolean inheritSW;    // should all child ECBs be SWITCHABLE as well?      //dynami2
     boolean useResClass;  // should this instance and its children use a       //dynami2 
                           //      resource class? (currently IBMBATCH)         //dynami2
     boolean useUtilClass;  // should this instance and its children use a      //dynami2 
                            //      utilization clas? (currently IBMBATCH)      //dynami2
     char utilClass[9];    // up to 8-character utilization class (plus null)   //dynami2
   };

   typedef struct __pool_start qfnl_pool_start_parm_list;
   typedef struct __pool_start *qfnl_pool_start_parm_list_Ptr;

#define __POOL_START
#endif

// The following structure is used to hold parameters on a ZTEST FFACt POOL STATUS command
#ifndef __POOL_STATUS
   struct __pool_status {
     int group;           // integer version of group number to display status
     char groupString[6]; // string version of group number
                          // its length is 6 = size of string 99999 which is 5 plus 1 for null terminator
   };

   typedef struct __pool_status qfnl_pool_status_parm_list;
   typedef struct __pool_status *qfnl_pool_status_parm_list_Ptr;

#define __POOL_STATUS
#endif

// The following structure is used to hold parameters on a ZTEST FFACt POOL LIST command
#ifndef __POOL_LIST
   struct __pool_list {
     int group;           // integer version of group number to display status
     char groupString[6]; // string version of group number
                          // its length is 6 = size of string 99999 which is 5 plus 1 for null terminator
     unsigned int number_records;  // number of records to list
   };

   typedef struct __pool_list qfnl_pool_list_parm_list;
   typedef struct __pool_list *qfnl_pool_list_parm_list_Ptr;

#define __POOL_LIST
#endif

// The following structure is used to hold parameters on a ZTEST FFACt POOL STOP command
#ifndef __POOL_STOP
   struct __pool_stop {
     int group;           // integer version of group number to stop
   };

   typedef struct __pool_stop qfnl_pool_stop_parm_list;
   typedef struct __pool_stop *qfnl_pool_stop_parm_list_Ptr;

#define __POOL_STOP
#endif

// The following structure is used to hold parameters on a ZTEST FFACt POOL KILL command
#ifndef __POOL_KILL
   struct __pool_kill {
     int group;           // integer version of group number to kill
     char groupString[6]; // string version of group number
                          // its length is 6 = size of string 99999 which is 5 plus 1 for null terminator
   };

   typedef struct __pool_kill qfnl_pool_kill_parm_list;
   typedef struct __pool_kill *qfnl_pool_kill_parm_list_Ptr;

#define __POOL_KILL
#endif

// The following structure is used to hold parameters on a ZTEST FFACt POOL KILL command
#ifndef __POOL_CLEAR
   struct __pool_clear {
     int group;           // integer version of group number whose counts should be cleared
   };

   typedef struct __pool_clear qfnl_pool_clear_parm_list;
   typedef struct __fixed_clear *qfnl_pool_clear_parm_list_Ptr;

#define __POOL_CLEAR
#endif

/* MEMORY Structure */

// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
//  Beginning of Structures used to keep track of Fixed File Groups

#define QFNL_FIXED_TOKEN "QFIX    "
#define QFNL_FIXED_OWNER "QFNL_FIXED    "
#define QFNL_MAX_FIXED 20
#define QFNL_NUM_FRAMES 1

#ifndef __QFNL_GROUP_STATS
   struct __qfnl_group_stats {
      long readCount;      // total number of FINDs to date
      long writeCount;     // total number of FILEs to date
      long holdCount;      // total number of holds to date
      long instanceCount;  // number of instances STARTed
      unsigned long ecbCreateCount; // total number of child ECBs that were created for this group to date (where each child ECB exits)
      unsigned long ecbExitCount;   // total number of child ECBs that have exited
      unsigned long ecbCount;       // total number of ECBs currently active
      long errCount;       // total number of errors, broken down into:
      long mallocErr;      // total number of malloc errors
      long envErr;         // total number of miscellaneous environmental errors
      long ioErr;          // total number of FIND/FILE errors
      long contentErr;     // total number of times record content is in error
      boolean lockingActive; // if there is currently an ECB doing locking, do not allow others to run without locking,
                             //  and vice versa (if there is an ECB active without locking, do not allow others to run with locking
   };
   typedef struct __qfnl_group_stats  qfnl_group_stats;
   typedef struct __qfnl_group_stats *qfnl_group_stats_Ptr;
#define __QFNL_GROUP_STATS 1
#endif

#ifndef __QFNL_FIXED_INFO
   struct __qfnl_fixed_info {
      char              ctltbl_ent_lock[8];                // used to serialize updates for fixed group info
      int               group_number;                      // number assigned to group
      char              rec_ID[QFNL_RID_SIZE];             // record ID for this group of fixed records
      char              rec_type[QFNL_RTYPE_SIZE];         // record type for this group of fixed records
      int               record_size;                       // size of records in this group
      unsigned int      begin_ordinal;                     // beginning ordinal
      unsigned int      end_ordinal;                       // ending ordinal
      int               group_flag;                        // used for status of this group
      qfnl_group_stats  group_stats;                       // stat counters for this group
   };
   typedef struct __qfnl_fixed_info  qfnl_fixed_info;
   typedef struct __qfnl_fixed_info * qfnl_fixed_info_Ptr;
#define QFNL_GROUP_STAT_STARTED 3
#define QFNL_GROUP_STAT_STOPPED 2
#define QFNL_GROUP_STAT_BUILDING 1
#define QFNL_GROUP_STAT_FREESLOT 0
#define __QFNL_FIXED_INFO 1
#endif

// Define the "FIXED File Control Table" which lists all FIXED FILE GROUPs in memory for this processor
#ifndef __QFNL_FIXED_CTL_TBL
   struct __qfnl_fixed_ctl_tbl {
      char eye_catcher[ITOKENLEN];
      int number_of_groups;
      qfnl_fixed_info my_fixed[QFNL_MAX_FIXED];
   };
   typedef struct __qfnl_fixed_ctl_tbl  qfnl_fixed_ctl_tbl;
   typedef struct __qfnl_fixed_ctl_tbl *qfnl_fixed_ctl_tbl_Ptr;
#define __QFNL_FIXED_CTL_TBL 1
#endif

//  END of Structures used to keep track of Fixed File Groups
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<


// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
//  Beginning of Structures used to keep track of Pool Groups

#define QFNL_POOL_OWNER "QFNL_POOL     "
#define QFNL_POOL_TOKEN "QPOO    "
#define QFNL_POOL_GENERIC_TOKEN "QPOOL   "
#define QFNL_MAX_POOL 20         // max number of pool groups per subsystem

#ifndef __QFNL_POOL_ITEM
   struct __qfnl_pool_item {
       tpf_TOD_ext_type tod_added;  // time the pool was added to the group
       tpf_TOD_ext_type tod_updated;  // time the pool was last filed, for possible verification
                                      //      (originally for expiry but not currently used)
       TPF_FA8 FA;  // the pool file address
       int flags; // for expiry, keep track if valid or not
#define QFNL_POOL_ITEM_NOT_VALID 0
#define QFNL_POOL_ITEM_VALID 1
   };
   typedef struct __qfnl_pool_item qfnl_pool_item;
   typedef struct __qfnl_pool_item * qfnl_pool_item_Ptr;
#define __QFNL_POOL_ITEM 1
#endif

#ifndef __QFNL_POOL_BATCH
   struct __qfnl_pool_batch {
      char groupString[6];  // eye catcher to see group this batch belongs to
      int  group_number;     // number assigned to group
      uint num_frames; // number of frames allocated for this batch
      char token[ITOKENLEN]; // token used on tpf_gsysc
      char first_item;  // first item in this group of pools
   };
   typedef struct __qfnl_pool_batch qfnl_pool_batch;
   typedef struct __qfnl_pool_batch * qfnl_pool_batch_Ptr;
#define __QFNL_POOL_BATCH 1
#endif

#ifndef __QFNL_POOL_INFO
   struct __qfnl_pool_info {
      char              ctltbl_ent_lock[8];                // used to serialize updates for fixed group info
      int               group_number;                      // number assigned to group
      unsigned int number_records;
      tpf_TOD_ext_type tod_built;   // time that this pool group was built (used for expiry)
      char hex_rid[QFNL_RID_SIZE];  // used to populate records
      char stpool[QFNL_STPOOL_SIZE];  // short term pool section for this group
      union {
        unsigned short getfc_rid;  // RID used by GETFC just to get from the appropriate pool section
        char handle_id[2];
      } handle;
      int record_size;     // we need to save this in case we are doing writes without reads
      int expiry; // number of seconds (60*input Expire value in minutes)
      qfnl_pool_batch_Ptr  pool_batch_ptr; // pointer to the "batch" of pools comprising this group
      int               group_flag;                        // used for status of this group
      qfnl_group_stats  group_stats;                       // stat counters for this group
   };
   typedef struct __qfnl_pool_info  qfnl_pool_info;
   typedef struct __qfnl_pool_info * qfnl_pool_info_Ptr;
#define __QFNL_pool_INFO 1
#endif

// Define the "POOL Group Control Table" which lists all POOL GROUPs
//   NOTE: POOL GROUPs are processor-unique
#ifndef __QFNL_POOL_CTL_TBL
   struct __qfnl_pool_ctl_tbl {
      char eye_catcher[ITOKENLEN];
      int number_of_groups;
      qfnl_pool_info my_pool[QFNL_MAX_POOL];
   };
   typedef struct __qfnl_pool_ctl_tbl  qfnl_pool_ctl_tbl;
   typedef struct __qfnl_pool_ctl_tbl *qfnl_pool_ctl_tbl_Ptr;
#define __QFNL_POOL_CTL_TBL 1
#endif

//  END of Structures used to keep track of POOL File Groups
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<

#ifndef __QFNL_DATA_HEADER

//  !!! NOTE !!!
//  This is the definition of the structure that
//   is used as a heading for each record (currently fixed or pool).
//  Please note:  You might need to keep the structures for
//     qfnl_data_header and qfnl_data_item consistent.
//     If you change one of these structures in format, you
//     might want to consider changing the other as well.
//

struct __qfnl_data_header {
    char              rec_ID[QFNL_RID_SIZE];
    char              rcc[QFNL_RCC_SIZE];
    char              program[4];
    char              FA[8];
    tpf_TOD_ext_type tod;
    int record_size;
    struct __eyecatcher {
      char cpuID;
      char repeat_cpuID[3];
    } eyecatcher;

};
typedef struct __qfnl_data_header qfnl_data_header;
typedef struct __qfnl_data_header * qfnl_data_header_Ptr;
#define __QFNL_DATA_HEADER 1
#endif

#ifndef __QFNL_DATA_ITEM

//  !!! NOTE !!!
//  This is the definition of the structure that
//   is used as a heading for each record (currently fixed or pool).
//  Please note:  You might need to keep the structures for
//     qfnl_data_header and qfnl_data_item consistent.
//     If you change one of these structures in format, you
//     might want to consider changing the other as well.
//

struct __qfnl_data_item {
    char              FA[8];
    tpf_TOD_ext_type tod;
    int counter;
};

typedef struct __qfnl_data_item qfnl_data_item;
typedef struct __qfnl_data_item * qfnl_data_item_Ptr;
#define __QFNL_DATA_ITEM 1
#endif

inline char thisCPUid(void)
{
  struct pi1dt *pidt;
  char cpuid;

  pidt = (struct pi1dt *) cinfc_fast(CINFC_CMMPID);
  cpuid = pidt->pi1pids.pi1cpui;

  return (cpuid);
}

/**********************************************************************/
/* Control Block                                                      */
/**********************************************************************/

typedef enum {
  INIT=0,
  START,
  STOP
} drvr_status;

typedef struct {
  char hdr[4];
  unsigned long lock;
  unsigned int instances;
  struct {
    drvr_status status;
    unsigned int instances;
    unsigned int tests_run;
    unsigned int tests_fail;
  } is[SAISTRM];
} drvr_cntl;

/**********************************************************************/
/* Continuous Mode Information Block                                  */
/**********************************************************************/
typedef struct {
  char hdr[6];
  short int casenum;
  int flag;
  int ecbroute;
} drvr_ecbinfo;
// TODO: check the information block for correct information needed
//       for invoking continuous mode.  This is used with a CREMC.

char * getSysHeapToken(char *);

/**********************************************************************/
/* Helper Functions                                                   */
/**********************************************************************/

void printMsg(int, char, enum t_wtopc_chain, char *, ...)                       //d26352
              __attribute__ ((format (printf, 4, 5)));                          //d26352

typedef enum {
  DISPHEAD,  /* First message in chain */
  DISPCONT,  /* Messages in chain      */
  DISPSTOP   /* Last message in chain  */
} infochain;

void printInfo(infochain, char *, ...)                                          //d26352
               __attribute__ ((format (printf, 2, 3)));                         //d26352
#define INFOMSG(fmt,args...) printInfo(DISPCONT, "  " fmt, ## args)

void printWarn(infochain, char *, ...)                                          //d26352
               __attribute__ ((format (printf, 2, 3)));                         //d26352
#define WARNMSG(fmt,args...) printWarn(DISPCONT, "  " fmt, ## args)

void printErr(int, char *, ...)                                                 //d26352
              __attribute__ ((format (printf, 2, 3)));                          //d26352
#define ERRMSG(fmt,args...) printErr(102, "CASE: %s (line %d)\n" fmt, \
                                     api, __LINE__, ## args)



// TODO: place your global functions here if needed

#endif

