/*********************************************************************/
/*      Licensed Materials - Property of IBM                         */
/*      "Restricted Materials of IBM"                                */
/*      5748-T15                                                     */
/*      Copyright IBM Corp. 2016                                     */
/*      All Rights Reserved                                          */
/*      US Government Users Restricted Rights - Use, duplication     */
/*      or dislosure restricted by GSA ADP Schedule Contract         */
/*      with IBM Corp.                                               */
/*********************************************************************/
/* This file is provided on an "as is" basis.  There are no          */
/* warranties, express or implied, including the implied warranties  */
/* of merchantability and fitness for a particular purpose.          */
/* IBM has no obligation to provide service, defect correction, or   */
/* any maintenance for the file.  IBM has no obligation to supply    */
/* any updates or enhancements for the file to you even if such are  */
/* or later become available.                                        */
/*********************************************************************/
/* Change History                                                    */
/*                                                                   */
/*  Defect           Date               Description                  */
/*  FFACT           02/01/16            Created                      */
/*                                                                   */
/*                                                                   */
/*********************************************************************/

#include "qfnl.h"

extern int repeat_canned;
extern int verbose;
extern int warnings;

/**********************************************************************/
/* Test Cases                                                         */
/*   Implement the test function in this area, then add the function  */
/*   to the 'testcases' variable. Each function must be               */
/*   self-contained, not relying on any prior test case for setup.    */
/*                                                                    */
/*   After writing the test function, add the function name to the    */
/*   test case wrangler function "runApi" at the bottom of this       */
/*   source file.                                                     */
/*                                                                    */
/*   Example function:                                                */
/*     static void functionName()                                     */
/*     {                                                              */
/*       static char api[] = "functionName";                          */
/*                                                                    */
/*       INFOMSG("Informative Text : %d", number);                    */
/*       ERRMSG("Error Message : %d 0x%x", number, ptr);              */
/*       ...                                                          */
/*     }                                                              */
/*                                                                    */
/*     The INFOMSG macro will check for verbose output and chain the  */
/*     message with other output for the test case. The ERRMSG macro  */
/*     will automatically tag the output with the 'api' and line      */
/*     number of the message, making tracking easier.                 */
/**********************************************************************/

/**********************************************************************/
/* Sample Test Case                                                   */
/**********************************************************************/
static int testCase1()
{
  static char api[] = "testCase1";
  int rc = DRVR_SUCCESS;

  INFOMSG("There is nothing coded for %s", api);

  // TODO: some logic goes here. All test cases in this template
  //       are self-contained. Individual test cases should not
  //       depend on a previous (or future) test cases.
  
  // TODO: Make sure to clean up all memory before returning!
  
  return rc;
}

/**********************************************************************/
/* Another Sample Test Case                                           */
/**********************************************************************/
static int testCase2()
{
  static char api[] = "testCase2";
  int rc;

  INFOMSG("There is nothing coded for %s", api);
  rc = DRVR_SUCCESS;
  return rc;

 //  rc = -1;
 //  ERRMSG("  Boom! rc=%d", rc);

 //  return DRVR_FAILURE;
}


/**********************************************************************/
/* Test Case Wrangler                                                 */
/**********************************************************************/
int runApi(int casenum)
{
  int rc;
  testfp testcases[] = { /* ALWAYS ADD TO THE END OF THE LIST */
    testCase1,                               /*  0 */
    testCase2,                               /*  1 */
    NULL   /* INSERT NEW CASES BEFORE THIS LINE */
  };
  int totalcases = (sizeof(testcases) / sizeof(testfp)) - 1;
  
  /********************************************************************/
  /* Execute ALL test cases (casenum == -1)                           */
  /********************************************************************/
  if (casenum < 0)
  {
    int i;
    
    for (i = 0; i < totalcases; i++)
    {
      printInfo(DISPHEAD, "%s TEST CASE %d", DRVR_HDR, i+1);
      rc = testcases[i]();
      printInfo(DISPSTOP, "END OF DISPLAY");

      dlayc();
    }
    
    return DRVR_SUCCESS;
  }

  /********************************************************************/
  /* Invalid test case                                                */
  /********************************************************************/
  if (casenum >= totalcases)
  {
    if (warnings)
      printErr(101, "%s TEST CASE %d NOT DEFINED", DRVR_HDR, casenum+1);
    
    return DRVR_INVALID;
  }

  /********************************************************************/
  /* NULL pointer                                                     */
  /********************************************************************/
  if (testcases[casenum] == NULL)
  {
    if (verbose)
      printErr(102, "%s TEST CASE %d IGNORED", DRVR_HDR, casenum+1);
    
    return DRVR_SUCCESS;
  }
  
  /********************************************************************/
  /* Execute a specific test case                                     */
  /********************************************************************/
  printInfo(DISPHEAD, "%s TEST CASE %d", DRVR_HDR, casenum+1);
  rc = testcases[casenum]();
  printInfo(DISPSTOP, "END OF DISPLAY");
  return rc;
}
